#########################################################################
##  VELOCITY.R
##  
##  This is a R program, written by Bruce E. Hansen.
##  It replicates Table 4 and Figures 1 and 2 from my paper
##  "The Grid Bootstrap and the Autoregressive Model".
##
##  Note: Package "pracma" is required. Run the following command in R first: 
##        install.packages("pracma")
##
##  If you have questions, please contact me at
##  
##  Bruce E. Hansen
##  Department of Economics
##  Social Science Building
##  University of Wisconsin
##  Madison, WI 53706-1393
##  behansen@wisc.edu
##  
#########################################################################

library("pracma")

g <- 200 	 # 200 #
b <- 1999    # 1999 #
bq <- 1999   # 1999 #
c <- .9      # confidence level #
width <- 6
if (c==.9)  aa <- 1.645
cc <- rbind(((1-c)/2),((c+1)/2))
rc <- as.matrix(rev(cc))

stock_t <- as.matrix(read.table("stock_t.dat"))  
np <- as.matrix(read.table("nplosser.dat"))  
y  <- as.matrix(np[1215:1334])
n <- nrow(y)

x11()
xs <- 1869
x <- as.matrix(seq(xs,xs+(n-1),1))
mtit <- "Velocity: 1869-1988"
plot(x,y,lty=1,col=1,type="l",ann=0)
title(main=mtit)

#*****************************************************************#

for (main in 1:1){
cat ("Nelson-Plosser Velocity","\n")
cat ("Sample:   1869-1988","\n")
cat ("Gridpoints                 ", g,"\n")
cat ("Standard Bootstraps        ", b,"\n")
cat ("Grid Bootstraps            ", bq,"\n")
cat ("Confidence Level           ", c,"\n")
cat ("\n")
determ <- cbind(matrix(1,(n-1),1),seq(1,n-1,1))
d0 <- rbind(c(1,0),determ)
yt <- y-d0%*%qr.solve(d0,y)
y0 <- yt[1]
xname <- rbind("Asymptotic","Stock","Efron","Perct-t","Kilian","Grid-a","Grid-t")
nt <- nrow(xname)

b_est <- function(yy){
  x <- cbind(determ,as.matrix(yy[1:(n-1)]))
  y <- as.matrix(yy[2:n])
  xx <- solve(t(x)%*%x)   
  theta <- xx%*%(t(x)%*%y)
  e <- y-x%*%theta
  se <- as.matrix(sqrt(diag(xx*as.vector(t(e)%*%e)/(n-4))))
list(theta=theta,se=se,e=e)
}

out <- b_est(y)
theta <- out$theta
se <- out$se
e <- out$e
ahat <- theta[3]
sehat <- se[3]
Ttheta <- format(theta,digits=3)
Tse <- format(se,digits=3)
cat ("Variable  ","Estimate  ","S.E.    ","\n")
cat ("Intercpt  ",Ttheta[1]," ",Tse[1],"\n")
cat ("Trend     ",Ttheta[2]," ",Tse[2],"\n")
cat ("Y(-1)     ",Ttheta[3]," ",Tse[3],"\n")


# Stock's proc for computing local-to unity confidence intervals #

unitci <- function(tdf,clevel,cv){
 alpha0 <- 0.5*(1-clevel)
 sel0 <- (trunc(1000*as.matrix(cv[1,])+.01)==fix(1000*alpha0+.01))
 sel1 <- (trunc(1000*as.matrix(cv[1,])+.01)==fix(1000*(1-alpha0)+.01))
 i0 <- sum(as.matrix(seq(1,ncol(cv),1))*sel0)
 i1 <- sum(as.matrix(seq(1,ncol(cv),1))*sel1)
 cc <- as.matrix(cv[2:nrow(cv),1])
 cv0 <- as.matrix(cv[2:nrow(cv),i0])
 cv1 <- as.matrix(cv[2:nrow(cv),i1])
 d1 <- (cv0<tdf) 
 d2 <- (cv1>tdf) 
 dd <- d1*d2

 # empty confidence interval -- outside the range covered by cv #
 if (sum(d1)==0){
     c0 <- -99.99
     c1 <- -99.99
 }
 if (sum(d2)==0){ 
     c0 <- 99.99 
     c1 <- 99.99
 }
 if (i0==i1){  # return median -- 0-width CI #
    if (sum(d1)>0 & sum(d2)>0){
       jj <- as.matrix(seq(1,nrow(cc),1))
       jj <- as.matrix(jj[d1>0])
       j0 <- max(jj) 
       j1 <- j0+1
       if (j0<nrow(cc)){
          wt0 <- (tdf-cv0[j0,1])/(cv0[(j0+1),1]-cv0[j0,1])
          c0 <- (1-wt0)*cc[j0,1] + wt0*cc[(j0+1),1]
       }else{ 
          wt0 <- 0 
          c0 <- 99.99 
       }
       if (j1>1){
          wt1 <- (tdf-cv1[(j1-1),1])/(cv1[j1,1]-cv1[(j1-1),1])
          c1 <- (1-wt1)*cc[(j1-1),1] + wt1*cc[j1,1]
       }else{ 
          wt1 <- 0 
          c1 <- -99.99 
       }
    }
    if (sum(d1)==0){ 
       c0 <- -99.99 
       c1 <- c0
    }
    if (sum(d2)==0){ 
       c0 <- 99.99 
       c1 <- c0 
    }
 }
 if (sum(dd)>0){ # nonempty CI #
    jj <- as.matrix(seq(1,nrow(cc),1))
    jj <- as.matrix(jj[dd>0])
    j0 <- max(jj) 
    j1 <- min(jj)
    if (j0<nrow(cc)){
       wt0 <- (tdf-cv0[j0,1])/(cv0[(j0+1),1]-cv0[j0,1])
       c0 <- (1-wt0)*cc[j0,1] + wt0*cc[(j0+1),1]
    }else{ 
       wt0 <- 0 
       c0 <- 99.99 
    }
    if (j1>1){
       wt1 <- (tdf-cv1[(j1-1),1])/(cv1[j1,1]-cv1[(j1-1),1])
       c1 <- (1-wt1)*cc[(j1-1),1] + wt1*cc[j1,1]
    }else{ 
       wt1 <- 0 
       c1 <- -99.99 
    }
 }
 rbind(c1,c0)
}

ends <- matrix(0,nt,2)

# Asymptotic methods #
ui <- ahat+rbind(-1,1)*sehat*aa
ends[1,] <- t(ui)
ui <- unitci((ahat-1)/sehat,c,stock_t)  # stock #
ends[2,] <- t(1+ui/(n-1))

ar1 <- function(y0,a,e){
  nn <- nrow(e)
  if (abs(a)<1) y00 <- y0 else y00 <- 0
  et <- as.matrix(e[ceiling(nn*runif(n))])
  y <- y00
  for (i in 2:nrow(et)){
       datbb <- et[i]
       datbb <- datbb+a*y[i-1]
       y <- rbind(y,datbb)
  }
  y
}

# Standard Bootstraps #
boota <- matrix(0,b,1)
boott <- matrix(0,b,1)
for (j in 1:b){
   out <- b_est(ar1(y0,ahat,e))
   thetab <- out$theta
   seb <- out$se
   boota[j] <- thetab[3]
   boott[j] <- (thetab[3]-ahat)/seb[3]
}

quantile <- function(x,c){
  b <- nrow(x) 
  r <- ncol(x)
  xx <- x
  for (j in 1:r) xx[,j] <- sort(x[,j])
  bc <- b*c
  i <- floor(bc)
  i <- i+(i == 0)
  q <- xx[i,]*((i==bc)%*%matrix(1,1,r))+((xx[i,]+xx[(i+1),])/2)*((i!=bc)%*%matrix(1,1,r))
  out <- t(q)
  out
}

# efron #
ef <- quantile(boota,cc)
ends[3,] <- ef

# percentile t #
qt <- quantile(boott,rc)
pt <- ahat-qt*sehat
ends[4,] <- pt

# Kilian bootstrap #
if (ahat>=1){
  bc <- ahat
  bias <- 0
}else{
  bias <- mean(boota)-ahat 
  bc <- ahat-bias
  if (bc>=1){
      bc <- 1
      bias <- ahat-bc
  }
}
for (j in 1:b){
   out <- b_est(ar1(y0,bc,e))
   thetab <- out$theta
   boota[j] <- thetab[3]-bias
}
ends[5,] <- quantile(boota,cc)

cross <- function(a,x,y){
  ms <- 0/0
  rn <- nrow(a)
  r <- (x<y)
  d <- rbind(0,as.matrix((r[2:rn]-r[1:(rn-1)])))
  s <- as.matrix(seq(1,rn,1))
  li <- as.matrix(s[(d == 1)>0])
  if (is.nan(max(li))==0){
    al <- as.matrix(a[li-1])
    au <- as.matrix(a[li])
    bl <- as.matrix(x[li-1]-y[li-1])   
    bu <- as.matrix(y[li]-x[li])
    g1 <- (bu*al+bl*au)/(bl+bu)
  }else{
    g1 <- ms
  }
  li <- as.matrix(s[(d == -1)>0])
  if (sum(d==-1)==0) li <- 0/0
  if (is.nan(max(li))==0){
    al <- as.matrix(a[li-1])
    au <- as.matrix(a[li])
    bl <- as.matrix(x[li-1]-y[li-1])   
    bu <- as.matrix(y[li]-x[li])
    g2 <- (bu*al+bl*au)/(bl+bu)
  }else{
    g2 <- ms
  }
  list(g1=g1,g2=g2)
}

g_end <- function(a0,ta,grid){
  out <- cross(a0,ta,as.matrix(grid[,1]))
  l1 <- out$g1
  l2 <- out$g2
  out <- cross(a0,ta,as.matrix(grid[,2]))
  u1 <- out$g1
  u2 <- out$g2
  s <- as.matrix(sort(rbind(l1,l2,u1,u2)))
  l <- min(s)
  u <- max(s)
  if (sum(is.nan(u2)==0)>0) if (l==min(u2)) l <- min(a0)
  if (sum(is.nan(l1)==0)>0) if (l==min(l1)) l <- min(a0)
  if (sum(is.nan(l2)==0)>0) if (u==max(l2)) u <- max(a0)
  if (sum(is.nan(u1)==0)>0) if (u==max(u1)) u <- max(a0)
  cbind(l,u)
}

epi <- function(q,a,h,leave){
  a <- a%*%matrix(1,1,nrow(a))
  u <- (a-t(a))/h
  w <- (1-u^2)*(abs(u)<= 1)
  if (leave==1) diag(w) <- matrix(0,nrow(a),1)
  w <- w/(as.matrix(colSums(t(w)))%*%matrix(1,1,nrow(a)))
  w%*%q
}

kq_reg <- function(y,x,c){
  hn <- 5
  cn <- nrow(c)
  hh <- as.matrix(seq(1,hn,1)*(x[2]-x[1]))
  check <- matrix(0,hn,cn)
  q <- quantile(y,c)
  for (hi in 1:hn){
    h <- hh[hi]
    if (x[2]-x[1]<h){
       qh <- epi(q,x,h,1)
       check[hi,] <- colSums((q-qh)^2)
    }else{
       check[hi,] <- colSums(q^2)
    }
  }
  hi <- matrix(0,cn,1)
  for (j in 1:cn) hi[j] <- which.min(check[,j])
  h <- as.matrix(hh[hi])
  qh <- matrix(0,g,cn)
  for (ci in 1:cn) qh[,ci] <- epi(q[,ci],x,h[ci],0)
  qh
}

# Grid Bootstrap #
bootq <- matrix(0,bq,g)
bootse <- matrix(0,bq,g)
a0 <- as.matrix(seq(.84,.84+(g-1)*(.22/g),.22/g))
for (ak in 1:g){
  alphak <- a0[ak]
  for (j in 1:bq){
     out <- b_est(ar1(y0,alphak,e))
     thetab <- out$theta
     seb <- out$se
     bootq[j,ak] <- thetab[3]    
     bootse[j,ak] <- seb[3]
  }
}
ba <- bootq-(matrix(1,bq,1)%*%t(a0))
ts <- ba/bootse
grid_p <- kq_reg(ba,a0,cc)
grid_t <- kq_reg(ts,a0,cc)
ah0 <- ahat-a0
ts <- ah0/sehat
ends[6,] <- g_end(a0,ah0,grid_p)
ends[7,] <- g_end(a0,ts,grid_t)

cat ("\n")
cat ("Confidence Intervals for AR(1) Parameter","\n")
cat ("Method     ","Left End    ","Right End ","\n")
Txname <- format(xname,digits=4)
Tends <- format(ends,digits=4)
for (j in 1:nt) cat(Txname[j]," ",Tends[j,1],"    ",Tends[j,2],"\n")
cat ("________________________________________________________________","\n")
cat ("\n")


# Plot 1 - Efron Construction #
x11()
z <- matrix(0,g,1)
eu <- ef[2]-ahat
el <- ef[1]-ahat
l <- ends[6,1]
u <- ends[6,2]
ql <- ahat-l
qu <- ahat-u
hpu <- ahat+eu
hpl <- ahat+el
mtit <- "Velocity: 1869-1988"
ytit <- "Bootstrap Quantile"
xtit <- "90% Percentile Confidence Interval"
xtics <- c(.84,1)
ytics <- c(-.15,.05)
plot(a0,(a0-ahat),lty=1,col=1,type="l",xlim=xtics,ylim=ytics,ann=0)
lines(a0,z,lty=1,col=1)
lines(a0,grid_p[,2],lty=2,col=4)
lines(a0,grid_p[,1],lty=2,col=4)  
title(main=mtit,ylab=ytit,xlab=xtit) 
points(ahat,0,pch=19)
points(ahat,eu,pch=22)
points(ahat,el,pch=22)
points(hpu,eu,pch=23)
points(hpl,el,pch=23)
lines(rbind(hpl,ahat),rbind(el,el),lty=3,col=2)
lines(rbind(ahat,ahat),rbind(el,eu),lty=3,col=2)
lines(rbind(ahat,hpu),rbind(eu,eu),lty=3,col=2)
lines(rbind(hpu,hpu),rbind(eu,0),lty=4,col=3)
lines(rbind(hpl,hpl),rbind(el,0),lty=4,col=3)
points(hpu,0,pch=25,col=3)
points(hpl,0,pch=24,col=3)
 
# Plot 2 - Grid-alpha Construction (Figure 2) #
x11()
z <- matrix(0,g,1)
eu <- ef[2]-ahat
el <- ef[1]-ahat
l <- ends[6,1]
u <- ends[6,2]
ql <- ahat-l
qu <- ahat-u
mtit <- rbind("Figure 2","Velocity: 1869-1988")
ytit <- "Bootstrap Quantile"
xtit <- "90% Grid-alpha Confidence Interval"
xtics <- c(.9,1.06)
ytics <- c(-.20,.05)
plot(a0,ah0,lty=1,col=1,type="l",xlim=xtics,ylim=ytics,ann=0)
lines(a0,z,lty=1,col=1)
lines(a0,grid_p[,2],lty=2,col=4)
lines(a0,grid_p[,1],lty=2,col=4)  
title(main=mtit,ylab=ytit,xlab=xtit)
points(ahat,0,pch=19)
points(l,ql,pch=22)
points(u,qu,pch=22)
lines(rbind(l,l),rbind(ql,0),lty=4,col=2)
lines(rbind(u,u),rbind(qu,0),lty=4,col=2)
points(l,0,pch=25,col=2)
points(u,0,pch=24,col=2) 


# Plot 3 - Percentile-t and Grid-t Construction (Figure 1) #
x11()
l <- ends[7,1]
u <- ends[7,2]
eu <- qt[1]
el <- qt[2]
ql <- (ahat-l)/sehat
qu <- (ahat-u)/sehat
hl <- ahat-sehat*el
hu <- ahat-sehat*eu
mtit <- rbind("Figure 1","Velocity: 1869-1988")
ytit <- "Bootstrap Quantile"
xtit <- "90% Percentile-t and Grid-t Confidence Intervals"
xtics <- c(.9,1.06)
ytics <- c(-4,2)
plot(a0,ts,lty=1,col=1,type="l",xlim=xtics,ylim=ytics,ann=0)
lines(a0,z,lty=1,col=1)
lines(a0,grid_t[,2],lty=2,col=4)
lines(a0,grid_t[,1],lty=2,col=4)  
title(main=mtit,ylab=ytit,xlab=xtit)
points(ahat,0,pch=19)
points(ahat,eu,pch=22)
points(ahat,el,pch=22)
points(hl,el,pch=23)
points(hu,eu,pch=23)
points(l,ql,pch=21)
points(u,qu,pch=21)
lines(rbind(hl,ahat),rbind(el,el),lty=3,col=2)
lines(rbind(ahat,ahat),rbind(el,eu),lty=3,col=2)
lines(rbind(ahat,hu),rbind(eu,eu),lty=3,col=2)
lines(rbind(l,l),rbind(ql,0),lty=4,col=3)
lines(rbind(u,u),rbind(qu,0),lty=4,col=3)
lines(rbind(hu,hu),rbind(eu,0),lty=4,col=6)
lines(rbind(hl,hl),rbind(el,0),lty=4,col=6)
points(l,0,pch=25,col=3)
points(u,0,pch=24,col=3)
points(hl,0,pch=24,col=6)
points(hu,0,pch=25,col=6)
}



